#include <stdlib.h>
#include <stdio.h>
#include <strings.h>

#define TRKLEN (53248)
#define TRKBYTES (TRKLEN/8)

unsigned char lut[256];

const unsigned char encodetab[64] ={
0x96, 0x97, 0x9a, 0x9b, 0x9d, 0x9e, 0x9f, 0xa6,
0xa7, 0xab, 0xac, 0xad, 0xae, 0xaf, 0xb2, 0xb3,
0xb4, 0xb5, 0xb6, 0xb7, 0xb9, 0xba, 0xbb, 0xbc,
0xbd, 0xbe, 0xbf, 0xcb, 0xcd, 0xce, 0xcf, 0xd3,
0xd6, 0xd7, 0xd9, 0xda, 0xdb, 0xdc, 0xdd, 0xde,
0xdf, 0xe5, 0xe6, 0xe7, 0xe9, 0xea, 0xeb, 0xec,
0xed, 0xee, 0xef, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6,
0xf7, 0xf9, 0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff	
};

const unsigned char skew[16]=  { 0,13,11,9,7,5,3,1,14,12,10,8,6,4,2,15 };
const unsigned char skewr[16]= { 0,7,14,6,13,5,12,4,11,3,10,2,9,1,8,15 };

void hdump(unsigned char *buf,int max)
{
	int i,j,k;
	
	for (i=0;i<256; i+=16) {
		printf("%03X  ",i);
		k=((max-i)>16)? 16 : max-i;
		for (j=0;j<k;j++) printf("%02X ",buf[i+j]);
		for (;j<16;j++) printf("   ");
		for (j=0;j<k;j++) putchar((buf[i+j]>=' ' && buf[i+j]<128)?buf[i+j]:'.');
		putchar('\n');
		if (k<16) break;
	}
}

unsigned char trkdata[TRKBYTES+2];
int bitix=0;

void serialize(int d, int nb)
{
	int i;
	for (i=0;i<nb;i++) {
		if (d&0x80) trkdata[bitix>>3]|=1<<(7-(bitix&7));
		d<<=1;
		bitix++;
	}
}

main(int argc, char **argv)
{
	int i,j,k,v,s,t,c,x,a,sz;
	unsigned char *p,buf[256],buf2[86],sectors[16*256];
	FILE *fp,*fpo;
	
	if (argc<3) {
		printf(
			"Usage: %s <input.dsk> <output.img>\n"
			" %s makes a raw Apple II floppy disk image from an emulator <.dsk> image\n"
			" The resulting image has 50000 bits per track and includes everything: \n"
			" data, address fields, syncing leaders, gaps... The input file only stores\n"
			" the useful data of the disk (32768 bits per track)\n"
			" Notice that <dsk> files have the sectors interleaved\n"
			"   (order: 0,13,11,9,7,5,3,1,14,12,10,8,6,4,2,15)\n"
			,argv[0],argv[0]);
		exit(1);
	}
	
	if ((fp=fopen(argv[1],"rb"))==NULL) exit(1);
	if ((fpo=fopen(argv[2],"wb"))==NULL) exit(1);
	
	v=254;	// volume
	
	for (i=0;i<35;i++) {
		bzero(trkdata,TRKBYTES);
		bitix=0;
		for (k=0;k<16;k++) serialize(0xFF,10);	// Preamble
		fread(sectors,1,16*256,fp);
		for(j=0;j<16;j++) {
			//fread(buf,1,256,fp);
			//s=skew[j]; t=i;
			s=j; t=i;
			memcpy(buf,&sectors[skewr[j]*256],256);
			//printf("T%3d S%3d (%d)\n",i,j,s);

			// Separate the 6bit from 2bit data
			for (k=0;k<86;k++) buf2[k]=0;
			for(k=x=0,c=86;k<256;k++) {
				if (--c<0) {c=85; x+=2;}				
				if (buf[k]&1) buf2[c]|=1<<(x+1);
				if (buf[k]&2) buf2[c]|=1<<x;
				buf[k]>>=2;
			}
			//hdump(buf,256); putchar('\n'); hdump(buf2,86);
			//exit(0);
			
			// Send address
			serialize(0xD5,8);
			serialize(0xAA,8);
			serialize(0x96,8);
			serialize((v>>1)|0xaa,8);	serialize(v|0xaa,8);
			serialize((t>>1)|0xaa,8);	serialize(t|0xaa,8);
			serialize((s>>1)|0xaa,8);	serialize(s|0xaa,8);
			c=v^t^s;
			serialize((c>>1)|0xaa,8);	serialize(c|0xaa,8);
			serialize(0xDE,8);
			serialize(0xAA,8);
			serialize(0xEB,8);
			//gap
			serialize(0x0,13);
			// Now the data
			for (k=0;k<5;k++) serialize(0xFF,10);	// Preamble
			serialize(0xD5,8);
			serialize(0xAA,8);
			serialize(0xAD,8);

			x=0;
			for (k=85;k>=0;k--) {	// 2bits
				a=buf2[k];
				serialize(encodetab[a^x],8);
				x=a;
			}
			
			for (k=0;k<256;k++) {	// 6bits
				a=buf[k];
				serialize(encodetab[a^x],8);
				x=a;
			}
			serialize(encodetab[a],8);	// checksum
			serialize(0xDE,8);
			serialize(0xAA,8);
			serialize(0xEB,8);
			//gap
			serialize(0x0,13);
			for (k=0;k<15;k++) serialize(0xFF,10);	// Preamble
		}
		sz=bitix;
		while (bitix<TRKLEN) serialize(0,8);
		fwrite(trkdata,1,TRKBYTES,fpo);
	}
	fclose(fp);
	fclose(fpo);
	printf("%s written (%d used bits out of %d per track)\n",argv[2],sz,TRKLEN);
	
}
