///////////////////////////////////////////////////////
// Apple II computer recreated
//  by Jesus Arias (2023)
//

module main(		// TOP module with actual FPGA pins
	output TXD, 
	input  RXD, 
	input  CLKIN, 
	output HSYN, 
	output VSYN, 
	output [3:0]R,
	output [3:0]G,
	output [3:0]B,
	inout  [15:0]XD,
	output [15:0]XA,
	output XWE,
	output XOE,
	output XBHE,
	output XBLE,
	
	input  MISO,
	output MOSI,
	output SCK,
	output SSBF,
	output SSBSD,
	
	output JPIN1,	// SCK2 
	output JPIN2, 	// MOSI2
	output JPIN3, 	// /CS2
	input  JPIN4,	// MISO
	
	output PWM,
	input  KCLK,
	input  KDAT,
	
	output [1:0]JP
);

wire clk4x,pll_lock;
//-- PLL instance (25MHz)
pll
  pll1(
	.clock_in(CLKIN),
	.clock_out(clk4x),
	.locked(pll_lock)
	);

//-- Apple II instance 

wire sck,sckin;

SYSTEM sys1(.txd(TXD), .rxd(RXD), .clk4x(clk4x), .reset(reset),
	    .hsyn(HSYN), .vsyn(VSYN), .video({R,G,B}),
		.xwe(XWE), .xoe(XOE), .xdo(xdo), .xdi(xdi), .xa(XA), .xbhe(XBHE), .xble(XBLE),
		.sck(SCK), .miso(MISO), .mosi(MOSI), .ss1(SSBF), .ss2(SSBSD), 
		.pwmout(PWM), .kclk(KCLK), .kdat(KDAT),
		.sck2(JPIN1), .mosi2(JPIN2), .ssb2(JPIN3), .miso2(JPIN4)
		//,.debug(JP)
);

// Internal RESET: 31 VSYN (~0.5sec)
wire reset;
reg [4:0]cnt=5'b11111;
assign reset=(cnt!=0);

always @(posedge VSYN) cnt<=reset ? cnt-1: cnt;


//////////////////////////////////////////////////////
// Tristate pins for external RAM data bus
wire oe;
wire [15:0]xdi;
wire [15:0]xdo;

assign oe=XOE; // caution: XOE active low => writes on high
assign XD = oe ? xdo :16'hzzzz;
assign xdi = XD;

endmodule


